@@ -123,9 +123,16 @@ $config['template_dirs'] = array(DOCROOT."/templates",DOCROOT."/templates.dist")
 /*
  * Default graph dimensions 
  */
-$config['graph_width']  = "500";
-$config['graph_height'] = "100";
-$config['zgraph_width']  = "500";
-$config['zgraph_height'] = "100";
-$config['pdf_width']    = "675";
-$config['pdf_height']   = "100";
+$config['graph_width']       = "500";
+$config['graph_height']      = "100";
+$config['zgraph_width']      = "500";
+$config['zgraph_height']     = "100";
+$config['pdf_width']         = "675";
+$config['pdf_height']        = "100";
+$config['right_zoom_offset'] = "30";
+$config['mobile_devices']    = "iPhone";
+$config['pdf_page_size']     = "A4";
+$config['pdf_margin_left']   = "17.5";
+$config['pdf_margin_right']  = "10";
+$config['pdf_margin_top']    = "30";
+
