@@ -370,7 +370,7 @@ class Data_Model extends System_Model
             }
         return;
         }
-        if( $view == ""){
+        if( $view === ""){
             $v = 0;
             foreach($this->config->views as $view_key=>$view_val){
                 $i=0;
@@ -746,6 +746,30 @@ class Data_Model extends System_Model
            $this->TIMERANGE = $timerange;
     }
 
+    public function buildBasketStruct($basket,$view = NULL){
+        if(is_array($basket) && sizeof($basket) > 0){
+	    if($view == ""){
+                 $views = array_keys($this->config->views);
+            }else{
+		 $views = array($view);
+            }
+            foreach($views as $view){
+                foreach($basket as $item){
+                    # explode host::service::source
+                    $slices = explode("::",$item);
+                    if(sizeof($slices) == 2)
+                        $this->buildDataStruct($slices[0], $slices[1], $view);
+                    if(sizeof($slices) == 3)
+                        $this->buildDataStruct($slices[0], $slices[1], $view, $slices[2]);
+                }
+                # break on custom time ranges
+		if(isset($this->TIMERANGE['type']) && $this->TIMERANGE['type'] == "start-end"){
+                    break;
+                }
+            }
+        }
+    }
+
     public function buildPageStruct($page,$view){
         $servicelist = array();
         $this->parse_page_cfg($page);
