@@ -7,9 +7,13 @@
 <?php
 foreach($services as $service){
 	$path = pnp::addToUri( array('host' => $host, 'srv' => $service['name']) );
+	echo pnp::add_to_basket_icon($host,
+                $service['name']);
+
 	echo "<a href=\"".$path."\" class=\"multi".$service['is_multi']. " " . $service['state'].
 		 "\" title=\"".$service['servicedesc']. 
-		 "\">".pnp::shorten($service['servicedesc']).
+		 "\">";
+	echo pnp::shorten($service['servicedesc']).
 		 "</a><br>\n";
 }
 ?>
